# Generated by Django 2.1.7 on 2019-05-03 08:05

from django.db import migrations, models
import nuntius.models


class Migration(migrations.Migration):

    dependencies = [("nuntius", "0013_esp_message_id_not_editable")]

    operations = [
        migrations.AddField(
            model_name="campaign",
            name="signature_key",
            field=models.BinaryField(
                default=nuntius.models.Campaign.generate_signature_key, max_length=20
            ),
        )
    ]
